--[[
************************************************************************

tracker.lua

File date: 2009-09-16T22:44:56Z
File revision: 225
Project revision: 225
Project version: r225


Author: Angelike

************************************************************************
]]--

local beql = LibStub("AceAddon-3.0"):GetAddon("beql")
local L = LibStub("AceLocale-3.0"):GetLocale("beql")
local WatchFrame = WatchFrame;

local frames = {}

local leftclicktime, rightclicktime = 0,0

function beql:InitTracker()
	beql:debugprint("Init QuestTracker")
--
--- Start Framemanagemant
--

	-- we use a own frame for quest, because other addons may want to use the WatchFrame
	-- Create a new watchframe
	frames.QuestWatchFrame = CreateFrame("Frame", "QuestWatchFrame", UIParent)
	frames.QuestWatchFrame:SetFrameStrata("BACKGROUND")
	frames.QuestWatchFrame:SetFrameLevel(2)
	frames.QuestWatchFrame:ClearAllPoints()
	frames.QuestWatchFrame:SetPoint("TOPLEFT","UIParent","Center",0,0)
	frames.QuestWatchFrame:SetAlpha(1)
	frames.QuestWatchFrame:SetWidth(200)
	frames.QuestWatchFrame:SetHeight(200)
	frames.QuestWatchFrame:Show()
	frames.QuestWatchFrame:SetMovable(true)
	frames.QuestWatchFrame:EnableMouse(true) 

	-- Handle Mouse events
	frames.QuestWatchFrame:SetScript("OnDragStart",function() 
		if not beql.db.char.saved.minimizedtracker then 
			QuestWatchFrame.ismoving = true 
			QuestWatchFrame:StartMoving() 
		end 
	end)
	frames.QuestWatchFrame:SetScript("OnDragStop",function() 
		QuestWatchFrame.ismoving = false 
		QuestWatchFrame:StopMovingOrSizing() 
		beql:SavePositionTracker() 
	end)

	frames.QuestWatchFrame:SetScript("OnMouseDown", function(self, button) 
		-- handel mouse down events, save time of the click
		if button == "LeftButton" then
			leftclicktime = GetTime()
		elseif button == "RightButton" then
			rightclicktime = GetTime()
		end
	end)
	
	frames.QuestWatchFrame:SetScript("OnMouseUp", function(self, button)
		if button == "LeftButton" and (GetTime() - leftclicktime) < 1 then
			if not beql.db.char.saved.minimizedtracker then 
				beql:TrackerClick() 
			end 
		elseif button == "RightButton" and (GetTime() - rightclicktime) < 1 then
			if not beql.db.char.saved.minimizedtracker then 
				beql:TrackerClick() 
			end 
		end
	end)
	
	frames.QuestWatchFrame:SetScript("OnUpdate", function() 
		if not MouseIsOver(QuestWatchFrame) and beql.db.char.saved.tooltipset then
			ResetCursor()
			GameTooltip:FadeOut()
			beql.db.char.saved.tooltipset = false
		end	
		if MouseIsOver(QuestWatchFrame) then 
			beql:TrackerTooltip() 
		end 
	end)

	frames.QuestWatchFrame:SetBackdrop({
--		bgFile="Interface\\Tooltips\\UI-Tooltip-Background",
--		edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile = true, edgeSize = 16, tileSize = 16,
--		insets = { left= 4, right = 4, top = 4, bottom = 4 }
		})

		-- check if tracker was placed
	if beql.db.char.saved.qtrackercorner and beql.db.char.saved.qtrackerposx and beql.db.char.saved.qtrackerposy then
		-- move it to old position
		frames.QuestWatchFrame:ClearAllPoints()
		frames.QuestWatchFrame:SetPoint(beql.db.char.saved.qtrackercorner,"UIParent","BOTTOMLEFT", beql.db.char.saved.qtrackerposx, beql.db.char.saved.qtrackerposy)
	else
		-- tracker wasn't placed, move to center of screen
		frames.QuestWatchFrame:ClearAllPoints()
		frames.QuestWatchFrame:SetPoint("TOPLEFT","UIParent","Center",0,0)
		beql:SavePositionTracker()
	end

	
	-- Create Linesframe
	frames.QuestWatchLines = CreateFrame("Frame","QuestWatchLines", QuestWatchFrame)
	frames.QuestWatchLines:ClearAllPoints()
	frames.QuestWatchLines:SetAllPoints(QuestWatchFrame)
	frames.QuestWatchLines:SetAlpha(1)
	frames.QuestWatchLines:Show()
	
	
	-- Create backdrop
	frames.WatchFrameBackdrop = CreateFrame("Frame","WatchFrameBackdrop", QuestWatchFrame)
	frames.WatchFrameBackdrop:ClearAllPoints()
	if IsAddOnLoaded("Mobmap") then
		frames.WatchFrameBackdrop:SetPoint("TOPLEFT", QuestWatchFrame, "TOPLEFT",-20, 5)
		frames.WatchFrameBackdrop:SetPoint("BOTTOMLEFT", QuestWatchFrame, "BOTTOMLEFT",-20,-5)
	else
		frames.WatchFrameBackdrop:SetPoint("TOPLEFT", QuestWatchFrame ,"TOPLEFT", -5, 5)
		frames.WatchFrameBackdrop:SetPoint("BOTTOMLEFT",QuestWatchFrame, "BOTTOMLEFT",-5,-5)
	end 
	frames.WatchFrameBackdrop:SetPoint("TOPRIGHT", QuestWatchFrame ,"TOPRIGHT", 5, 5)	
	frames.WatchFrameBackdrop:SetPoint("BOTTOMRIGHT",QuestWatchFrame, "BOTTOMRIGHT", 5,-5) 
	frames.WatchFrameBackdrop:SetFrameStrata("BACKGROUND")
	
	frames.WatchFrameBackdrop:SetBackdrop({
		bgFile="Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile = true, edgeSize = 16, tileSize = 16,
		insets = { left= 4, right = 4, top = 4, bottom = 4 }
		})
	frames.WatchFrameBackdrop:SetFrameLevel("1")
	frames.WatchFrameBackdrop:Show()	
	frames.WatchFrameBackdrop:SetMovable(true)

	
	-- create titlebar for the tracker
	frames.TrackerTitleBar = CreateFrame("Frame","TrackerTitleBar", QuestWatchFrame)
	frames.TrackerTitleBar:ClearAllPoints()
	frames.TrackerTitleBar:SetHeight(32)
	if IsAddOnLoaded("Mobmap") then
		frames.TrackerTitleBar:SetPoint("TOPLEFT",QuestWatchFrame, "TOPLEFT",-20,32)
	else
		frames.TrackerTitleBar:SetPoint("TOPLEFT",QuestWatchFrame, "TOPLEFT",-5,32)
	end
	frames.TrackerTitleBar:SetPoint("TOPRIGHT",QuestWatchFrame, "TOPRIGHT",5,32)
	frames.TrackerTitleBar:SetBackdrop({
		bgFile="Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile = true, edgeSize = 16, tileSize = 16,
		insets = { left= 4, right = 4, top = 4, bottom = 4 }
		})			
	frames.TrackerTitleBar:SetFrameStrata("BACKGROUND")
	frames.TrackerTitleBar:SetFrameLevel("1")
	frames.TrackerTitleBar:SetAlpha(1)
	frames.TrackerTitleBar:SetMovable(true)
	frames.TrackerTitleBar:EnableMouse(true)
	frames.TrackerTitleBar:SetScript("OnDragStart",function() QuestWatchFrame:StartMoving() end)
	frames.TrackerTitleBar:SetScript("OnDragStop",function() QuestWatchFrame:StopMovingOrSizing() beql:SavePositionTracker() end)		
	frames.TrackerTitleBar:Show()

	
	-- create text for titlebar
	frames.TrackerTitleBarText = TrackerTitleBar:CreateFontString("TrackerTitleBarText","BACKGROUND","GameFontNormal")
	frames.TrackerTitleBarText:SetPoint("LEFT",10,1)
	frames.TrackerTitleBarText:SetAlpha(1)
	frames.TrackerTitleBarText:SetText("bEQL Dummytext")

	
	-- create minimizebutton in titlebar
	frames.TrackerMinimizeButton = CreateFrame("Button","TrackerMinimizeButton", TrackerTitleBar, "UIPanelButtonTemplate")
	frames.TrackerMinimizeButton:SetNormalTexture("Interface\\AddOns\\beql\\Images\\minimize_up")
	frames.TrackerMinimizeButton:SetPushedTexture("Interface\\AddOns\\beql\\Images\\minimize_down")
	frames.TrackerMinimizeButton:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	frames.TrackerMinimizeButton:SetWidth(18)
	frames.TrackerMinimizeButton:SetHeight(18)
	frames.TrackerMinimizeButton:Show()
	frames.TrackerMinimizeButton:ClearAllPoints()
	frames.TrackerMinimizeButton:SetPoint("TOPRIGHT",TrackerTitleBar, "TOPRIGHT", -4,2)
	frames.TrackerMinimizeButton:SetScript("OnClick", function() 
		if beql.db.char.saved.minimizedtracker then 
			beql:TrackerMaximize() 
		else 
			beql:TrackerMinimize() 
		end 
	end)



	WatchFrame_Update()
	beql:debugprint("Init QuestTracker - done")
end

-- minimize questtracker
function beql:TrackerMinimize()
	beql.db.char.saved.minimizedtracker = true
	frames.TrackerMinimizeButton:SetNormalTexture("Interface\\AddOns\\beql\\Images\\restore_up")
	frames.TrackerMinimizeButton:SetPushedTexture("Interface\\AddOns\\beql\\Images\\restore_down")	
	
	-- Hide Lines
	frames.QuestWatchLines:Hide();
	
	local minWidth = frames.TrackerTitleBarText:GetWidth()+32
	frames.QuestWatchFrame:SetHeight(1)
	frames.QuestWatchFrame:SetWidth(minWidth)
	frames.WatchFrameBackdrop:Hide()
end

-- maximze questtracker
function beql:TrackerMaximize()
	beql.db.char.saved.minimizedtracker = false
	frames.TrackerMinimizeButton:SetNormalTexture("Interface\\AddOns\\beql\\Images\\minimize_up")
	frames.TrackerMinimizeButton:SetPushedTexture("Interface\\AddOns\\beql\\Images\\minimize_down")	
	
	frames.QuestWatchLines:Show()
	frames.WatchFrameBackdrop:Show()
	WatchFrame_Update()
end

-- Toggles Questtracker status between active and inactive
function beql:ToggleQuestTracker(inactive)
	beql.db.profile.disabledtracker = inactive
	
	if(not inactive) then
		-- load tracker
		beql:debugprint("load questtracker")
		if not (QuestWatchFrame) then
			beql:InitTracker()
		end

		-- Callbacks
		beqlQ.RegisterCallback(beql, "Quest_Gained", beql.Quixote_Quest_Gained)
		beqlQ.RegisterCallback(beql, "Update", beql.Quixote_Update)
		beqlQ.RegisterCallback(beql, "Ready", beql.Quixote_Ready)
		beqlQ.RegisterCallback(beql, "Quest_Lost", beql.Quixote_Quest_Lost)
		
		WatchFrame_RemoveObjectiveHandler(WatchFrame_DisplayTrackedQuests);
		WatchFrame_AddObjectiveHandler(beql.DisplayTrackedQuests);
		
		-- make some hooks
		beql:RawHook("AddQuestWatch", "Hooks_AddQuestWatch", true)
		beql:RawHook("RemoveQuestWatch","Hooks_RemoveQuestWatch", true)
		beql:RawHook("IsQuestWatched", "Hooks_IsQuestWatched", true)
		beql:RawHook("GetNumQuestWatches", "Hooks_GetNumQuestWatches", true)
		beql:RawHook("GetQuestIndexForWatch", "Hooks_GetQuestIndexForWatch", true)
		
		QuestWatchFrame:Show()
		
		WATCHFRAME_MAXQUESTS = 25;
		MAX_WATCHABLE_QUESTS = 25;
		
		beql:debugprint("questracker loaded")
	elseif( inactive and QuestWatchFrame) then
		-- ask QuestWatchFrame to prevent unload on unloaded tracker
		
		-- unregister callbacks
		beqlQ.UnregisterCallback(beql, "Quest_Gained");
		beqlQ.UnregisterCallback(beql, "Update");
		beqlQ.UnregisterCallback(beql, "Ready");
		beqlQ.UnregisterCallback(beql, "Quest_Lost");
		
		-- unload tracker
		WatchFrame_RemoveObjectiveHandler(beql.DisplayTrackedQuests);
		WatchFrame_AddObjectiveHandler(WatchFrame_DisplayTrackedQuests);
		
		-- unhookfunctions
		if beql:IsActive() then -- only unhook on active addon
			--beql:Unhook("WatchFrame_Update");
			beql:Unhook("AddQuestWatch");
			beql:Unhook("RemoveQuestWatch");
			beql:Unhook("IsQuestWatched");
			beql:Unhook("GetNumQuestWatches");
			beql:Unhook("GetQuestIndexForWatch");
		end

		QuestWatchFrame:Hide();
		
		WATCHFRAME_MAXQUESTS = 10;
		MAX_WATCHABLE_QUESTS = 10;

		if not WatchFrame:IsVisible() then
			WatchFrame:Show()
		end
	end

	QuestLog_Update();
	WatchFrame_ClearDisplay();
	WatchFrame_Update(WatchFrame);

end

--
-- Events
--

function beql:Quixote_Ready()
	WatchFrame_Update()
end

function beql:Quixote_Update()
	WatchFrame_Update()
end

function beql:Quixote_Quest_Gained( questname, qID, objectives, category, provider, player)
	local _ , id, _ ,_ , _ ,_ , complete ,_ , _ ,_ = beqlQ:GetQuestByUid(qID)
	
	if(complete == 1) and qID == beql.db.char.newquestid then
		beql:Quixote_Quest_Complete(questname, uid)
	end
	
	if beql.db.profile.AddNew and qID == beql.db.char.newquestid then
		AddQuestWatch(id)
		QuestLog_Update()
		WatchFrame_Update()
	end
	beql.WatchQueue = nil
end

function beql:Quixote_Quest_Lost( qname, quid)
	WatchFrame_Update()
end


function beql:CHAT_MSG_SYSTEM(arg1)
	if beql.db.profile.AddNew then
		if string.find(arg1, BEQL_QUEST_ACCEPTED.." .+") then
			local temp = string.gsub(arg1, BEQL_QUEST_ACCEPTED.." ", "")
			beql.TrackerQueue = temp
			beql:ManageQuests()
		end
	end
end

--
-- Hooks
--

function beql:Hooks_AddQuestWatch(questIndex)
	local questName, level = GetQuestLogTitle(questIndex)
	local questLogHeader, tempId

	if IsQuestWatched(questIndex) then
		return
	else
		isHeader = false
		tempId = questIndex
		while (not isHeader) do
			questLogHeader, _, _, _, isHeader = GetQuestLogTitle(tempId)
			tempId = tempId-1
		end

		table.insert(beql.db.char.QuestWatches, questLogHeader..","..level..","..questName)
	end
	beql:SortWatchedQuests()
--	beql.hooks.AddQuestWatch(questIndex)
end

function beql:Hooks_RemoveQuestWatch(questIndex)
	if questIndex <= 0 then
		return
	end
	local questName, level = GetQuestLogTitle(questIndex)

	local questLogHeader, isHeader, tempId

	isHeader = false
	tempId = questIndex
	while (not isHeader) do
		questLogHeader, _, _, _, isHeader = GetQuestLogTitle(tempId)
		tempId = tempId-1
	end

	local temp = questLogHeader..","..level..","..questName

	if table.getn(beql.db.char.QuestWatches) > 0 then
		for i=1, table.getn(beql.db.char.QuestWatches) do
			if beql.db.char.QuestWatches[i] == temp then
				table.remove(beql.db.char.QuestWatches , i)
				break
			end
		end
	end
	beql:SortWatchedQuests()
	
--	beql.hooks.RemoveQuestWatch(questIndex)
end

function beql:Hooks_IsQuestWatched(questIndex)
	
	local questName, level = beql.hooks.GetQuestLogTitle(questIndex)
	local questLogHeader, isHeader, tempId
	local _

	isHeader = false
	tempId = questIndex
	while (not isHeader and tempId > 0) do
		questLogHeader, _, _, _, isHeader = beql.hooks.GetQuestLogTitle(tempId)
		tempId = tempId-1
	end
	

	local temp = questLogHeader..","..level..","..questName
	if table.getn(beql.db.char.QuestWatches) > 0 then
		for i=1, table.getn(beql.db.char.QuestWatches) do
			if beql.db.char.QuestWatches[i] == temp then
				return true
			end
		end
	end
	return false
end


-- Returns the number of watched quests
function beql:Hooks_GetNumQuestWatches()
	return table.getn(beql.db.char.QuestWatches)
end

function beql:Hooks_GetQuestIndexForWatch(id)
	local numEntries, numQuests = GetNumQuestLogEntries()
	local questLogTitleText, level
	local questLogHeader, isHeader, tempId
	local questFound = false
	local temp, currentHeader=nil
	local _

	for i=1, numEntries, 1 do
		questLogTitleText, level, _, _, isHeader, _ = GetQuestLogTitle(i)
		if isHeader then
			currentHeader = questLogTitleText
		else
			temp = currentHeader..","..level..","..questLogTitleText
			if  temp == beql.db.char.QuestWatches[id]  then
				return i
			end
		end
	end
	return 0
end

function beql:Hooks_WatchFrame_Update() 

	-- Hide empty WatchFrame
	if 	-- Timer
		select('#',GetQuestTimers()) == 0 
	and -- Achievements
		((select('#', GetTrackedAchievements()) == 0 and not beql.db.profile.atracker.Enable) or beql.db.profile.atracker.Enable) 
	and -- Quests
		((GetNumQuestWatches() == 0 and beql.db.profile.disabledtracker) or not beql.db.profile.disabledtracker) 
	then
		WatchFrame:Hide()
	else
		WatchFrame:Show()
	end
end

function beql:Hooks_WatchFrame_Update_old()
	-- Dont do anything before Quixote gets his data
--[[	if not beqlQ.firstScanDone then
		beqlQ.firstScanDone = true
		return
	end--]]
	-- Is it realy needed ?
	beql:ManageQuests()

	local questIndex,isRemoved,isCollapsed
	local qTitle,qLvl,qTag,qRec,qStat,qObj,qZone,qID, qdaily
	local oText, oType, oNumP, oNumN, oComp, numObjectives, objText
	local temp,curZone
	local watchTextID = 1
	local Color = {}
	local tempwidth = 0
	local tempColor,tempColor2,tempColor3
	local markerID = 0
	local completedQuests = 0
	local questWatchMaxWidth = 0	
	local _
	
	local minheight = 128
	
	--beql.db.char.TrackerQuests = {}
	
	for i=1, GetNumQuestWatches() do
		questIndex = GetQuestIndexForWatch(i)
		qID = nil;
		if questIndex then 
			questLogTitleText, _, _, _, _, _ = GetQuestLogTitle(questIndex)
--			print(GetQuestLogTitle(questIndex))
--			print(beqlQ:GetQuestById(questIndex))
			-- Q2 	return q.unique_id, q.id, q.title, q.level, q.tag, GetNumQuestLeaderBoards(q.id), q.complete, q.group, q.daily, q.zone
			-- Q1	return q.title, q.level, q.tag, q.group, q.complete, #q.leaderboard, q.zone, q.id
			-- Fix submitted by Ben Schreiber for MarsQuestOrganizer
			if questLogTitleText then
				--print(questLogTitleText)
				--qTitle, qLvl, qTag, qRec, qStat, qObj, qZone, qID = beqlQ:GetQuestByName(questLogTitleText)
				qID, _ , qTitle, qLvl, qTag, qObj, qStat, qRec, qdaily, qZone = beqlQ:GetQuestById(questIndex) -- _ is questIndex in Questlog
				--print(qTag)
			end
		end
	
		if qID then
			isRemoved = false
			
			if beql.db.profile.RemoveFinished and qStat == 1 then
				isRemoved = true
				completedQuests = completedQuests +1
			end					
			
			--
			-- Zone !
			--
			if (curZone == nil or curZone ~= qZone) and not isRemoved then
				curZone = qZone
				if beql.db.profile.ShowZonesInTracker then
					if beql.db.profile.CustomZoneColor then
						tempwidth = beql:AddWatchLineText(watchTextID,curZone,beql.db.profile.TrackerFontHeight,beql.db.profile.Color.Zone, beql.db.profile.TrackerFontHeight+2)
					else
						Color = {r = 0.8,g = 0.8,b = 1}
						tempwidth = beql:AddWatchLineText(watchTextID,curZone,beql.db.profile.TrackerFontHeight, Color, beql.db.profile.TrackerFontHeight+1)
					end
					if not(tempwidth == nil) and tempwidth > questWatchMaxWidth then 
						questWatchMaxWidth = tempwidth 
					end
					watchTextID = watchTextID + 1
				end
			end
				
			--
			-- Quest !
			--
			if qID > 0 and not isRemoved then
				if qStat == 1 then
					--Completed
					completedQuests = completedQuests +1
					if beql.db.profile.CustomHeaderColor then
						Color = beql.db.profile.Color.HeaderComplete
					else
						Color = NORMAL_FONT_COLOR
					end
				elseif qStat == -1 then
					-- Failed
					if beql.db.profile.CustomHeaderColor then
						Color = beql.db.profile.Color.HeaderFailed
					else
						Color = NORMAL_FONT_COLOR
					end
				else
					-- Normal
					if beql.db.profile.CustomHeaderColor then
						if beql.db.profile.HeaderQuestLevelColor then
							Color = GetQuestDifficultyColor(qLvl)
						else
							Color = beql.db.profile.Color.HeaderEmpty
						end
					else
						Color = { r = 0.75, g = 0.61, b = 0 }
					end
				end

				if beql.db.profile.showlevel then
					if(qTag == nil) then
						qTag = ""
					end
					tempwidth = beql:AddWatchLineText(watchTextID,"  ".."["..qLvl..string.upper(qTag).."] "..qTitle,beql.db.profile.TrackerFontHeight, Color)
				else
					tempwidth = beql:AddWatchLineText(watchTextID,"  "..qTitle,beql.db.profile.TrackerFontHeight, Color)				
				end

				--beql.db.char.TrackerQuests[watchTextID] = qID

				if not tempwidth == nil and tempwidth > questWatchMaxWidth then 
					questWatchMaxWidth = tempwidth 
				end
				watchTextID = watchTextID + 1
				
				--
				-- Objectives !
				--
				numObjectives = qObj
				if beql.db.profile.MinimizeFinished and qStat == 1 then
					numObjectives = 0
				end				
				if beql.db.profile.HideCompletedOnly and qStat == 1 then
					numObjectives = 0
				end
				markerID = 0

				if  numObjectives > 0 then

					for j=1, numObjectives do
						print(qID .. j)
						oText, oType, oNumP, oNumN, oComp = beqlQ:GetQuestStatusById(qID, j)
						-- Hack: MarsQuestOrganizer doesnt throw errors !!! Still is a fix needed !!! <- Quixote doesnt returns the correct number and/or the correct objectives for the current quest.
						if oNumN == nil then oNumN = 1 end				
						if oNumP == nil then oNumP = 1 end
						if oText == nil then oText = "" end
													
						if beql.db.profile.CustomObjetiveColor then
									tempColor = { r=beql.db.profile.Color.ObjectiveEmpty.r, g=beql.db.profile.Color.ObjectiveEmpty.g, b=beql.db.profile.Color.ObjectiveEmpty.b }
									tempColor2 = { r=beql.db.profile.Color.ObjectiveComplete.r, g=beql.db.profile.Color.ObjectiveComplete.g, b=beql.db.profile.Color.ObjectiveComplete.b }
						else
									tempColor = {r=0.8, g=0.8, b=0.8}
									tempColor2 = {r=HIGHLIGHT_FONT_COLOR.r, g=HIGHLIGHT_FONT_COLOR.g, b=HIGHLIGHT_FONT_COLOR.b}						
						end
						if oComp then
							Color = tempColor2
						else								
							if beql.db.profile.FadeColor then
								if oType == "reputation" then
									if oNumN == oNumP then
										Color = beql:FadeColors(tempColor, tempColor2, 1,1)
									else
										Color = beql:FadeColors(tempColor, tempColor2, 0,1)
									end
								else
									Color = beql:FadeColors(tempColor, tempColor2, oNumP,oNumN)
								end
							else
								Color = tempColor
							end
						end
						
						if  beql.db.profile.ShowObjectiveMarkers  then
							if beql.db.profile.UseTrackerListing then 
								objText = "   "..BEQL_TrackerLists[beql.db.profile.TrackerList][markerID]..") " 
							else
								objText = "   "..BEQL_TrackerSymbols[beql.db.profile.TrackerSymbol].." "
							end
						else
							if IsAddOnLoaded("Mobmap") then
								objText = " - "
							else
								objText = "   "
							end						
						end
											
						objText = objText..oText
						if oType == "item" or oType == "monster" or oType == "reputation" then
							objText = objText..": "..oNumP.."/"..oNumN
						end
						if (oComp and not beql.db.profile.MinimizeFinished) or (not oComp) then
							tempwidth = beql:AddWatchLineText(watchTextID,objText,beql.db.profile.TrackerFontHeight,Color)
							if tempwidth > questWatchMaxWidth then questWatchMaxWidth = tempwidth end
							watchTextID = watchTextID + 1
						end
						marekrID = markerID + 1
					end

				end
				
			end 
		end
	end 
	

	--Hide the tracker if nothing is watched or Show it and resize it
	watchTextID = 20 -- just 4 debug
	if ( watchTextID == 1 ) then
		--Set the tracker indicator
		QuestLogTrackTracking:SetVertexColor(1.0, 0, 0)
		QuestWatchFrame:Hide()
		return
	else
		-- Set the tracker indicator
		QuestLogTrackTracking:SetVertexColor(0, 1.0, 0)
		QuestWatchFrame:Show()
		QuestWatchFrame:SetHeight((watchTextID * beql.db.profile.TrackerFontHeight+1) + (watchTextID * 4) + 12)
		
		-- Auto-Resize the tracker to fit or set fixed width
		if not beql.db.profile.TrackerAutoResize then
			QuestWatchFrame:SetWidth(beql.db.profile.TrackerFixedWidth)
		else
			QuestWatchFrame:SetWidth(questWatchMaxWidth + 12)
			-- If the tracker is smaller than the minwidht, resize it
			local minWidth = beql.TrackerTitleBarText:GetWidth()+32
			local curWidth = QuestWatchFrame:GetWidth()
			if curWidth < minWidth then
				QuestWatchFrame:SetWidth(minWidth)
			end	
		end
	end


	--Position the Tracker
--	UIParent_ManageFramePositions()
	--Lock Tracker Corner for resizing
	beql:TrackerLockCornerForGrowth()
		
	
	local shown = 0
	table.foreach(beql.QuestLines, function(k,v)
		local watchframeline = beql.QuestLines[k]
		if(watchframeline:IsVisible()) then
			shown = shown + 1
			print(watchframeline:GetName() .. " shown")
		else
				print(watchframeline:GetName() .. " hidden")
		end
	end)
	

	if(shown == 0) then
		print("Hide Quest Tracker, because there is nothing 2 watch");
--		WatchFrame:Hide();
	else
		print("Something 2 watch, show Tracker (" .. WatchFrame.lineCache.numFrames-1 .. " Lines)");
--		WatchFrame:Show();
		local tmpheight = ((WatchFrame.lineCache.numFrames * beql.db.profile.TrackerFontHeight+1) + (WatchFrame.lineCache.numFrames * 4) + 12)
		if (tmpheight > 128) then
			print("Setze Trackerhöhe auf " .. tmpheight)
			-- WatchFrame:SetHeight(tmpheight)
		end
	end
	
end

local WATCHFRAMELINES_FONTSPACING = 2

function beql:DisplayTrackedQuests ()
	-- Don't do anything before Quixote gets his data
	if not beqlQ.firstDone then
		return 0,0
	end

	-- beql locales
	local questIndex,isRemoved,isCollapsed
	local qTitle,qLvl,qTag,qRec,qStat,qObj,qZone,qID, qdaily
	local oText, oType, oNumP, oNumN, oComp, numObjectives, objText
	local temp,curZone
	local zoneid = 0
	local Color = {}
	local tempwidth = 0
	local tempColor,tempColor2,tempColor3
	local markerID = 0
	local completedQuests = 0
	local questWatchMaxWidth = 0	
	local _
	local qObjectives, oItems
	
	local minheight = 128
	
	local zoneoffset = 5
	local questoffset = 1.5
	
	beql.db.char.TrackerQuests = {}

	-- blizz locales
	local lineFrame, maxHeight, frameWidth = QuestWatchLines, 600, 200
	local _;
	local text, finished;
	local questTitle;
	local watchItemIndex = 0;

	local line;
	local lastLine;
	local HIGHLIGHT_FONT_COLOR = HIGHLIGHT_FONT_COLOR;
	local NORMAL_FONT_COLOR = NORMAL_FONT_COLOR;
	local WATCHFRAMELINES_FONTHEIGHT = beql.db.profile.TrackerFontHeight
	local linkButton;
	
	local heightNeeded = 0;
	local maxWidth = 0;
	local questwidth = 0
	
	local iconHeightLeft = 0;
	local questLogTitleText
	
	local maxtrackerwidth, maxheaderwidth, maxiconwidth
	
	if not beql.db.profile.TrackerAutoResize then
		maxtrackerwidth = beql.db.profile.TrackerFixedWidth
		if ( SHOW_QUEST_OBJECTIVES_ON_MAP == "1" ) then
			maxheaderwidth = beql.db.profile.TrackerFixedWidth - WATCHFRAME_MAP_BUTTON_WIDTH - 8
		else
			maxheaderwidth = beql.db.profile.TrackerFixedWidth
		end
		
		
	end
	
	for i = 1, GetNumQuestWatches() do
		questWidth = 0;
		questIndex = GetQuestIndexForWatch(i);

		qID = nil;
		if questIndex then
			beql.AddTags = true
			questLogTitleText = GetQuestLogTitle(questIndex)
			beql.AddTags = nil
			-- Fix submitted by Ben Schreiber for MarsQuestOrganizer
			if questLogTitleText then
				qID, _ , qTitle, qLvl, qTag, qObj, qStat, qRec, qdaily, qZone = beqlQ:GetQuestById(questIndex)
				-- _ is questIndex in Questlog
			end
		end
			
		if qID then
			isRemoved = false
			
			if beql.db.profile.RemoveFinished and qStat == 1 then
				isRemoved = true
				completedQuests = completedQuests +1
			end
			
			--
			-- Zone !
			--
			if beql.db.profile.ShowZonesInTracker and (curZone == nil or curZone ~= qZone) and not isRemoved then
				curZone = qZone

				line = beql:GetQuestLine(lineFrame:GetName());
				line.text:SetText(curZone);
				if ((line.text:GetStringWidth() + 10) > questWatchMaxWidth) then
					questWatchMaxWidth = line.text:GetStringWidth() +10
				end
				line:Show();
				if ( not lastLine ) then -- First line
					line:SetPoint("TOPRIGHT", lineFrame, "TOPRIGHT", 0, 0);
					line:SetPoint("TOPLEFT", lineFrame, "TOPLEFT", 0, 0);
				else
					local yOffset = 0;
					if ( iconHeightLeft > 0 ) then
						yOffset = -iconHeightLeft;
					end
					
					line:SetPoint("TOPRIGHT", lastLine, "BOTTOMRIGHT", 0, yOffset - zoneoffset);
					line:SetPoint("TOPLEFT", lastLine, "BOTTOMLEFT", 0, yOffset - zoneoffset);
				end
				
				if beql.db.profile.CustomZoneColor then
					Color = beql.db.profile.Color.Zone
				else
					Color = {r = 0.8,g = 0.8,b = 1}
				end
				
				line.text:SetTextColor(Color.r, Color.g, Color.b)
				if not(tempwidth == nil) and tempwidth > questWatchMaxWidth then 
					questWatchMaxWidth = tempwidth 
				end
				
				maxWidth = max(maxWidth, line.text:GetStringWidth())
				lastLine = line;
			end
				
			--
			-- Quest !
			--
			if qID > 0 and not isRemoved then
				-- Colores
				if qStat == 1 then
					--Completed
					completedQuests = completedQuests +1
					if beql.db.profile.CustomHeaderColor then
						if beql.db.profile.HeaderQuestLevelColor then
							Color = GetQuestDifficultyColor(qLvl)
						else
							Color = beql.db.profile.Color.HeaderComplete
						end
					else
						Color = NORMAL_FONT_COLOR
					end
				elseif qStat == -1 then
					-- Failed
					if beql.db.profile.CustomHeaderColor then
						Color = beql.db.profile.Color.HeaderFailed
					else
						Color = NORMAL_FONT_COLOR
					end
				else
					-- Normal
					if beql.db.profile.CustomHeaderColor then
						if beql.db.profile.HeaderQuestLevelColor then
							Color = GetQuestDifficultyColor(qLvl)
						else
							Color = beql.db.profile.Color.HeaderEmpty
						end
					else
						Color = { r = 0.75, g = 0.61, b = 0 }
					end
				end
				
				qTitle = questLogTitleText;-- Levels

				if beql.db.profile.ShowZonesInTracker then
					-- add space for zone offset
					qTitle = "  " .. qTitle
				end

				line = beql:GetQuestLine(lineFrame:GetName());
				questTitle = line;
				line.text:SetText(qTitle);
				line.qID = qID;
				line.text:SetTextColor(Color.r, Color.g, Color.b)
				line:Show();
				if ( not lastLine ) then -- First line
					line:SetPoint("TOPRIGHT", lineFrame, "TOPRIGHT", 0, 0);
					line:SetPoint("TOPLEFT", lineFrame, "TOPLEFT", 0, 0);
				else
					local yOffset = 0;
					if ( iconHeightLeft > 0 ) then
						yOffset = -iconHeightLeft;
					end
					
					line:SetPoint("TOPRIGHT", lastLine, "BOTTOMRIGHT", 0, yOffset - questoffset);
					line:SetPoint("TOPLEFT", lastLine, "BOTTOMLEFT", 0, yOffset - questoffset);
				end
				iconHeightLeft = 0;
				local stringWidth = line.text:GetStringWidth();
				
				--
				-- ItemButton
				--
				local item, charges = select(2,GetQuestLogSpecialItemInfo(questIndex));
				local itemButton;
				if ( item ) then
					watchItemIndex = watchItemIndex + 1;
					itemButton = _G["beqlItem"..watchItemIndex];
					if ( not itemButton ) then
						beql.QuestItems = watchItemIndex
						itemButton = CreateFrame("BUTTON", "beqlItem" .. watchItemIndex, lineFrame, "WatchFrameItemButtonTemplate");
					end
					itemButton:Show();
					itemButton:ClearAllPoints();
					itemButton:SetID(questIndex);
					SetItemButtonTexture(itemButton, item);
					SetItemButtonCount(itemButton, charges);
					itemButton.charges = charges;
					WatchFrameItem_UpdateCooldown(itemButton);
					itemButton.rangeTimer = -1;
					if ( SHOW_QUEST_OBJECTIVES_ON_MAP ~= "1" ) then
						line.text:SetPoint("RIGHT", itemButton, "LEFT", -4, 0);
					end
					iconHeightLeft = WATCHFRAME_QUEST_WITH_ITEM_HEIGHT - WATCHFRAMELINES_FONTHEIGHT - WATCHFRAMELINES_FONTSPACING; -- We've already displayed a line for this
					if ( SHOW_QUEST_OBJECTIVES_ON_MAP == "1" ) then
						itemButton.maxStringWidth = stringWidth + WATCHFRAME_MAP_BUTTON_WIDTH;
					else
						itemButton.maxStringWidth = stringWidth
					end
				end
				
				lastLine = line;
				
				--
				-- Map Button (not active now, but maybe in future)
				-- 
				if ( SHOW_QUEST_OBJECTIVES_ON_MAP == "1" ) then
					stringWidth = stringWidth + WATCHFRAME_MAP_BUTTON_WIDTH;
					line.mapButton:Show();
					line.mapButton.questID = qID;
					line.text:ClearAllPoints();
					line.text:SetPoint("TOPLEFT");
					line.text:SetPoint("BOTTOM");
					local lineWidth = line:GetWidth();
					if maxtrackerwidth and ( line.text:GetWidth() + 4 + WATCHFRAME_MAP_BUTTON_WIDTH > maxtrackerwidth ) then -- 25 = button (22) + Buttonofset (3)
						line.text:SetWidth(maxtrackerwidth - (4 + WATCHFRAME_MAP_BUTTON_WIDTH));
					end
				end
				questWidth = max(stringWidth, questWidth);
				
				--
				-- Objectives
				-- 
				
				numObjectives = qObj
				if (beql.db.profile.MinimizeFinished or beql.db.profile.HideCompletedOnly) and qStat == 1 then
					-- hide questobjectives on complete quest
					numObjectives = 0;
				end

				markerID = 0
				
				
				local _, qObjectives = beqlQ:IterateObjectivesForQuest(qID)
				if  numObjectives > 0 and qObjectives then
					for oText, oItems in pairs(qObjectives) do
						oComp = false;
						oNumP, oNumN, oType = unpack(oItems)
						if oNumN == nil then oNumN = 1 end
						if oNumP == nil then oNumP = 1 end
						if oText == nil then oText = "" end

						if oNumP == oNumN then
							oComp = true
						end
						
						-- only display complete
						if (oComp and not beql.db.profile.MinimizeFinished) or (not oComp) then
						
							-- colores
							if beql.db.profile.CustomObjetiveColor then
										tempColor = { r=beql.db.profile.Color.ObjectiveEmpty.r, g=beql.db.profile.Color.ObjectiveEmpty.g, b=beql.db.profile.Color.ObjectiveEmpty.b }
										tempColor2 = { r=beql.db.profile.Color.ObjectiveComplete.r, g=beql.db.profile.Color.ObjectiveComplete.g, b=beql.db.profile.Color.ObjectiveComplete.b }
							else
										tempColor = {r=0.8, g=0.8, b=0.8}
										tempColor2 = {r=HIGHLIGHT_FONT_COLOR.r, g=HIGHLIGHT_FONT_COLOR.g, b=HIGHLIGHT_FONT_COLOR.b}
							end
							if oComp then
								Color = tempColor2
							else
								if beql.db.profile.FadeColor then
									if oType == "reputation" then
										if oNumN == oNumP then
											Color = beql:FadeColors(tempColor, tempColor2, 1,1)
										else
											Color = beql:FadeColors(tempColor, tempColor2, 0,1)
										end
									else
										Color = beql:FadeColors(tempColor, tempColor2, oNumP,oNumN)
									end
								else
									Color = tempColor
								end
							end
							
							-- markers
							if  beql.db.profile.ShowObjectiveMarkers  then
								if beql.db.profile.UseTrackerListing then 
									objText = "   "..BEQL_TrackerLists[beql.db.profile.TrackerList][markerID]..") " 
									
								else
									objText = "   "..BEQL_TrackerSymbols[beql.db.profile.TrackerSymbol].." "
								end
							else
								if IsAddOnLoaded("Mobmap") then
									objText = " - "
								else
									objText = "   "
								end
							end

							if beql.db.profile.ShowZonesInTracker then
								-- add space for zone offset
								objText = "  " .. objText
							end
							
							objText = objText..oText
							if oType == "item" or oType == "monster" or oType == "reputation" then
								objText = objText ..": "..oNumP.."/"..oNumN
							end

							-- create line
							line = beql:GetQuestLine(lineFrame:GetName());
							line.text:SetText(objText);
							line.qID = qID;
							line.text:SetTextColor(Color.r, Color.g, Color.b)
							line:SetPoint("TOPRIGHT", lastLine, "BOTTOMRIGHT", 0, WATCHFRAMELINES_FONTSPACING);
							line:SetPoint("TOPLEFT", lastLine, "BOTTOMLEFT", 0, WATCHFRAMELINES_FONTSPACING);
							line:Show();
							stringWidth = line.text:GetStringWidth();
							questWidth = max(stringWidth, questWidth);
							if ( iconHeightLeft > 0 ) then
								line.text:SetPoint("RIGHT", itemButton, "LEFT", -4, 0);
								itemButton.maxStringWidth = max(stringWidth, itemButton.maxStringWidth)
							end
							lastLine = line;
							iconHeightLeft = iconHeightLeft - WATCHFRAMELINES_FONTHEIGHT - WATCHFRAMELINES_FONTSPACING;
--							if tempwidth > questWatchMaxWidth then 
--								questWatchMaxWidth = tempwidth 
--							end
							--beql.db.char.TrackerQuests[(beql.FrameCache.QuestLineIndex - 1)] = qID
							markerID = markerID + 1
						end

					end
				end
				
				
				-- adjust item button
				local itemButtonXOffset = 8;
--				if ( SHOW_QUEST_OBJECTIVES_ON_MAP == "1" ) then
--					questWidth = questWidth-- + WATCHFRAME_MAP_BUTTON_WIDTH;
					--itemButtonXOffset = itemButtonXOffset + WATCHFRAME_MAP_BUTTON_WIDTH;
--				end
				
				if ( itemButton ) then
					questWidth = questWidth + WATCHFRAME_ITEM_WIDTH + itemButtonXOffset;
					if maxtrackerwidth then
						-- we have a absolute tracker width
						itemButtonXOffset = min(itemButton.maxStringWidth + itemButtonXOffset, maxtrackerwidth - WATCHFRAME_ITEM_WIDTH);
					else
						itemButtonXOffset = itemButton.maxStringWidth + itemButtonXOffset;
					end
					itemButton:SetPoint("TOPLEFT", questTitle, "TOPLEFT", itemButtonXOffset, -WATCHFRAMELINES_FONTSPACING);
					itemButton:Show();
					if ( SHOW_QUEST_OBJECTIVES_ON_MAP == "1" and (questTitle.text:GetWidth() + WATCHFRAME_MAP_BUTTON_WIDTH) > itemButtonXOffset ) then
						questTitle.text:SetPoint("RIGHT", itemButton, "LEFT", -WATCHFRAME_MAP_BUTTON_WIDTH, 0);
					end
				end
				maxWidth = max(questWidth, maxWidth);

			end
		end
		
	end
	
	-- hide unused buttons
	for i = watchItemIndex + 1, beql.QuestItems do
		_G["beqlItem" .. i]:Hide();
	end
	
	-- Handle Frame Visibility
	if beql.FrameCache.QuestLineIndex > 1 then
		frames.QuestWatchFrame:Show()
	else
		frames.QuestWatchFrame:Hide()
	end
	
	beql:ReleaseUnusedQuestLines();
	beql:ReleaseUnusedQuestButtons();
	
	if not frames.QuestWatchFrame:IsVisible() then
		-- frame is empty and hidden, exit
		return 0,0
	end
	
	-- QuestWatchFrame

	-- Show Tracker header if set, set quest count in tracker header
	if beql.db.profile.ShowTrackerHeader then
		t1, _, t2 = WatchFrameLine1Text:GetFont();
		frames.TrackerTitleBarText:SetFont(t1, beql.db.profile.TrackerFontHeight+2, t2)
		frames.TrackerTitleBarText:SetHeight(beql.db.profile.TrackerFontHeight+2)	
		frames.TrackerTitleBarText:Show()
		local questnums = GetNumQuestWatches()
		local _,questEntries = GetNumQuestLogEntries()
		if beql.db.profile.RemoveFinished then
			frames.TrackerTitleBarText:SetText(L["Quest Tracker"].. " ("..questnums - completedQuests.."/"..questEntries..")")		
		else
			frames.TrackerTitleBarText:SetText(L["Quest Tracker"].. " ("..questnums.."/"..questEntries..")")				
		end
		frames.TrackerMinimizeButton:Show()
		frames.TrackerTitleBar:Show()
	else
		frames.TrackerTitleBar:Hide()
		frames.TrackerMinimizeButton:Hide()
		if beql.db.char.saved.minimizedtracker then
			beql:TrackerMaximize()
		end
	end

	-- Tracker Background handling
	if beql.db.profile.CustomBackgroundColor then
		frames.WatchFrameBackdrop:SetBackdropColor(beql.db.profile.Color.BackgroundColor.r,beql.db.profile.Color.BackgroundColor.g,beql.db.profile.Color.BackgroundColor.b,beql.db.profile.Color.BackgroundColor.a)
		frames.WatchFrameBackdrop:SetBackdropBorderColor(beql.db.profile.Color.BackgroundCornerColor.r,beql.db.profile.Color.BackgroundCornerColor.g,beql.db.profile.Color.BackgroundCornerColor.b,beql.db.profile.Color.BackgroundCornerColor.a)
		frames.TrackerTitleBar:SetBackdropColor(beql.db.profile.Color.BackgroundColor.r,beql.db.profile.Color.BackgroundColor.g,beql.db.profile.Color.BackgroundColor.b,beql.db.profile.Color.BackgroundColor.a)
		frames.TrackerTitleBar:SetBackdropBorderColor(beql.db.profile.Color.BackgroundCornerColor.r,beql.db.profile.Color.BackgroundCornerColor.g,beql.db.profile.Color.BackgroundCornerColor.b,beql.db.profile.Color.BackgroundCornerColor.a)		
	else
		frames.WatchFrameBackdrop:SetBackdropBorderColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
		frames.WatchFrameBackdrop:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
		frames.TrackerTitleBar:SetBackdropBorderColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
		frames.TrackerTitleBar:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)					
	end
	
	if beql.db.profile.BackgroundAlpha then
		frames.WatchFrameBackdrop:SetAlpha(50)
	else
		frames.WatchFrameBackdrop:SetAlpha(0)
		frames.TrackerTitleBar:SetBackdropColor(0,0,0,0)
		frames.TrackerTitleBar:SetBackdropBorderColor(0,0,0,0)
	end

	--Minimize the Tracker if its state is set to minimized
	if beql.db.char.saved.minimizedtracker then
		beql:TrackerMinimize()
		return 0,0
	end
	
	-- set Tracker Width
	if not beql.db.profile.TrackerAutoResize then
		frames.QuestWatchFrame:SetWidth(beql.db.profile.TrackerFixedWidth)
	else
		--if maxWidth > (TrackerTitleBarText:GetWidth() + 32) then
			frames.QuestWatchFrame:SetWidth(maxWidth);
		--else
			--frames.QuestWatchFrame:SetWidth(TrackerTitleBarText:GetWidth() + 32);
		--end
	end

	-- Tracker height
	frames.QuestWatchFrame:SetHeight((beql.FrameCache.QuestLines[1]:GetTop() - beql.FrameCache.QuestLines[#beql.FrameCache.QuestLines]:GetBottom()) + 5)
	
	-- last item is an icon, expand size to it
	if ( iconHeightLeft > 0 ) then
		frames.QuestWatchFrame:SetHeight(frames.QuestWatchFrame:GetHeight() + iconHeightLeft + 2)
		--yOffset = -iconHeightLeft;
	end

	
	-- we use a own Frame, so we return a used hight of zero
	return 0, maxWidth;
end

function beql:GetQuestLinkButton (frameparent)
	local button = beql.FrameCache.QuestLinkButtons[beql.FrameCache.QuestLinkButtonIndex]
	if ( not button ) then
		beql.FrameCache.QuestLinkButtons[beql.FrameCache.QuestLinkButtonIndex] = beql.FrameCache.Buttons:GetFrame();
		button = beql.LinkButtons[beql.FrameCache.QuestLinkButtonIndex];
	end

	button:SetParent(frameparent)
	beql.FrameCache.QuestLinkButtonIndex = beql.FrameCache.QuestLinkButtonIndex + 1;
	return button;
end

function beql:ReleaseUnusedQuestButtons ()
	if beql.FrameCache.QuestButtonIndex == 1 then
		-- no buttons ever drawen -> exit
		return
	end
	
	local watchButton
	for i = beql.FrameCache.QuestLinkButtonIndex, #beql.FrameCache.QuestLinkButtons do
		watchButton = beql.LinkButtons[i];
		watchButton.type = nil
		watchButton.index = nil;
		watchButton:Hide();
		watchButton.frameCache:ReleaseFrame(watchButton);
		beql.LinkButtons[i] = nil;
	end
	
	beql.FrameCache.QuestLinkButtonIndex = 1
end

function beql:GetQuestLine (frameparent)

	local line = beql.FrameCache.QuestLines[beql.FrameCache.QuestLineIndex];
	if ( not line ) then
		beql.FrameCache.QuestLines[beql.FrameCache.QuestLineIndex] = beql.FrameCache.Lines:GetFrame();
		line = beql.FrameCache.QuestLines[beql.FrameCache.QuestLineIndex];
	end

	line:Reset();
	line:SetParent(frameparent);
	
	local t1,_ , t2 = line.text:GetFont()
	line.text:SetFont(t1, beql.db.profile.TrackerFontHeight, t1)
	line:SetHeight(beql.db.profile.TrackerFontHeight + WATCHFRAMELINES_FONTSPACING)

	beql.FrameCache.QuestLineIndex = beql.FrameCache.QuestLineIndex + 1;
	return line;
end

function beql:ReleaseUnusedQuestLines ()
	local line
	for i = beql.FrameCache.QuestLineIndex, #beql.FrameCache.QuestLines do
		line = beql.FrameCache.QuestLines[i];
		line:Hide();
		line.frameCache:ReleaseFrame(line);
		beql.FrameCache.QuestLines[i] = nil;
	end
	
	beql.FrameCache.QuestLineIndex = 1;
end

-- For adding Text to Watcher
function beql:AddWatchLineText(id,text,size,color,height)
--	local tmp = 0
--	if tmp == 0 then
--		return
--	end

	local curWidth, tempWidth
	local _
	--local linenr
	if size == nil then size = beql.db.profile.TrackerFontHeight end
	if color == nil then color = {r=1,g=1,b=1} end
	if height == nil then height = beql.db.profile.TrackerFontHeight end
	watchText = getglobal("QuestWatchLine"..id)
	if watchText ~= nil then
		t1, _, t2 = watchText:GetFont()
		watchText:SetWidth(0)
		watchText:SetHeight(0)
		watchText:SetText(text)
		watchText:SetNonSpaceWrap(true)
		watchText:SetFont(t1, size, t2)
		tempWidth = watchText:GetWidth()
		curWidth = watchText:GetStringWidth()
		if not beql.db.profile.TrackerAutoResize then
			if curWidth > beql.db.profile.TrackerFixedWidth then 
				-- Switching back to auto calculate so this is not needed
				--linenr = ceil(curWidth / beql.db.profile.TrackerFixedWidth)
				--watchText:SetHeight(height*linenr)
				watchText:SetWidth(beql.db.profile.TrackerFixedWidth)
			else
				watchText:SetHeight(height)
				watchText:SetWidth(curWidth)
			end
		else
			watchText:SetHeight(height)
			watchText:SetWidth(curWidth)
		end

		if ( id > 1 ) then
			watchText:SetPoint("TOPLEFT", "QuestWatchLine"..(id - 1), "BOTTOMLEFT", 0, -4)
		end
		watchText:SetTextColor(color.r, color.g, color.b)
		watchText:Show()
	else
		print("Frame QuestWatchLine"..id.." not found!")
	end
	return tempWidth
end


function beql:SavePositionTracker()
	local Left = QuestWatchFrame:GetLeft()
	local Right = QuestWatchFrame:GetRight()
	local Top = QuestWatchFrame:GetTop()
	local Bottom = QuestWatchFrame:GetBottom()
	local lock
	local pointone
	local pointtwo
	local TOPBOTTOM_MEDIAN = 384
	local LEFTRIGHT_MEDIAN = 512
	if Left and Right and Top and Bottom then
		if Bottom < TOPBOTTOM_MEDIAN and Top > TOPBOTTOM_MEDIAN then
			local topcross = Top - TOPBOTTOM_MEDIAN
			local bottomcross = TOPBOTTOM_MEDIAN - Bottom
			if bottomcross > topcross then
				lock = "BOTTOM"
				pointtwo = Bottom
			else
				lock = "TOP"
				pointtwo = Top
			end
		elseif Top > TOPBOTTOM_MEDIAN then
			lock = "TOP"
			pointtwo = Top
		elseif Bottom < TOPBOTTOM_MEDIAN then
			lock = "BOTTOM"
			pointtwo = Bottom
		end
		if Left < LEFTRIGHT_MEDIAN and Right > LEFTRIGHT_MEDIAN then
			local leftcross = LEFTRIGHT_MEDIAN - Left
			local rightcross = Right - LEFTRIGHT_MEDIAN
			if rightcross > leftcross then
				lock = lock.."RIGHT"
				pointone = Right
			else
				lock = lock.."LEFT"
				pointone = Left
			end
		elseif Left < LEFTRIGHT_MEDIAN then
			lock = lock.."LEFT"
			pointone = Left
		elseif Right > LEFTRIGHT_MEDIAN then
			lock = lock.."RIGHT"
			pointone = Right
		end
		if lock and lock ~= "" and pointone and pointtwo then
			QuestWatchFrame:ClearAllPoints()
			QuestWatchFrame:SetPoint(lock,"UIParent","BOTTOMLEFT",pointone,pointtwo)
			beql.db.char.saved.qtrackercorner = lock
			beql.db.char.saved.qtrackerposx = pointone
			beql.db.char.saved.qtrackerposy = pointtwo
		elseif beql.db.char.saved.qtrackercorner and beql.db.char.saved.qtrackerposx and beql.db.char.saved.qtrackerposy then
			QuestWatchFrame:ClearAllPoints()
			QuestWatchFrame:SetPoint(beql.db.char.saved.qtrackercorner,"UIParent","BOTTOMLEFT",beql.db.char.saved.qtrackerposx,beql.db.char.saved.qtrackerposy)
		end
	end
end

-- EOF --